/*-
 * Copyright (c) 2006. Ivan Voras <ivoras@gmail.com>
 * Released under the Artistic License
 */

/*
 * sqlcached benchmarking client
 * $Id: client_bench.c,v 1.4 2006/05/12 23:32:28 ivoras Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/un.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <readline/readline.h>
#include <readline/history.h>

#include "common.h"
#include "sqlcached_client.h"

static char* unix_socket_name = DEFAULT_UNIX_SOCKET;

int argc;
char **argv;

struct sc_client_conn sc;

static double time_float() {
    struct timeval tv;
    gettimeofday(&tv, NULL);
    return tv.tv_sec + (double)tv.tv_usec / 1000000.0;
}

int main(int _argc, char** _argv) {
    char *msg;
    int i, count, t;
    double time1, time2;
    struct sc_result_set rs;
    char cmd[200];
    
    argc = _argc;
    argv = _argv;

    if (sc_connect_unix(unix_socket_name, &sc, &msg) != SC_OK) {
        fprintf(stderr, "Error connecting: %s\n", msg);
        exit(1);
    }
    
    count = 100000;

    if (sc_exec_query(&sc, "DELETE FROM cache", NULL, NULL) != SC_OK) {
        fprintf(stderr, "Error deleting cache: %s\n", msg);
        exit(1);
    }

    printf("Timing %d INSERTs...\n", count);
    t = time(NULL);
    time1 = time_float();
    for (i = 0; i < count; i++) {
        sprintf(cmd, "INSERT INTO cache (key, value, time) VALUES ('key%d', 'value %d', %d)", i, i, t);
        if (sc_exec_query(&sc, cmd, &rs, &msg) != SC_OK) {
            fprintf(stderr, "error in INSERT: %s\n", msg);
            exit(1);
        }
    }
    time2 = time_float(NULL);

    printf("%0.1lf qps\n", (double)count / (time2 - time1));


    printf("Timing %d SELECTs...\n", count);
    t = time(NULL);
    time1 = time_float();
    for (i = 0; i < count; i++) {
        sprintf(cmd, "SELECT value FROM cache WHERE key= 'key%d'", i);
        if (sc_exec_query(&sc, cmd, &rs, &msg) != SC_OK) {
            fprintf(stderr, "error in SELECT: %s\n", msg);
            exit(1);
        }
    }
    time2 = time_float(NULL);

    printf("%0.1lf qps\n", (double)count / (time2 - time1));

    
    return 0;
}

