<?php
/**
 * SQLCacheD Client for PHP
 * Copyright (c) 2006. Ivan Voras
 * Released under the BSD License
 *
 * So, how slow is PHP?
 * The native C client can do ~ 16000 queries per second
 * to the standard sqlcached, but this client can do ~ 11000.
 * (But it was much easier to write :) )
 */

class SQLCacheD_Exception extends Exception {}

class SQLCacheD_Client {

    const DEFAULT_UNIX_SOCKET = '/tmp/sqlcached.sock';
    const SC_VER_SIG = 'sqlcached-1';
    const SOCK_UNIX = 'unix';
    const SOCK_TCP = 'tcp';

    private $sock;

    function __construct($conn = null) {
        if ($conn === null)
            $conn = 'unix://'.SQLCacheD_Client::DEFAULT_UNIX_SOCKET;
        $this->sock = fsockopen($conn, 0, $errno, $errmsg);
        if ($this->sock === false)
            throw new SQLCacheD_Exception($errmsg, $errno);
        fwrite($this->sock, 'VER '.SQLCacheD_Client::SC_VER_SIG."\r\n");
        $resp = trim(fgets($this->sock));
        if ($resp != "+VER ".SQLCacheD_Client::SC_VER_SIG)
            throw new SQLCacheD_Exception("Invalid version signature: $resp");
    }


    function query($sql) {
        fwrite($this->sock, 'SQL '.addcslashes($sql, "\000\r\n")."\r\n");
        fflush($this->sock);
        $resp = trim(fgets($this->sock));
        if (sscanf($resp, '+REC %d, %d', $n_rows, $n_cols) === 2) {
            if ($n_rows == 0)
                return array();
            $cols = array();
            for ($i = 0; $i < $n_cols; $i++)
                $cols[] = trim(fgets($this->sock));
            $rs = array();
            for ($i = 0; $i < $n_rows; $i++) {
                $rec = array();
                for ($j = 0; $j < $n_cols; $j++) {
                    $rec[$cols[$j]] = rtrim(fgets($this->sock));
                }
                $rs[] = $rec;
            }
            return $rs;
        } else if (sscanf($resp, "+NREC %d", $nrec) === 1) {
            return $nrec;
        }
        throw new SQLCacheD_Exception("Error: $resp in ".addslashes($sql));
    }


    function set_expiry($table, $interval, $ops = 0, $recs = 0) {
        $table = addcslashes($table, "\000\r\n");
        $interval = intval($interval);
        $ops = intval($ops);
        $recs = intval($recs);
        fwrite($this->sock, "EXP $table $interval $ops $recs\r\n");
        fflush($this->sock);
        $resp = trim(fgets($this->sock));
        if ($resp[0] == '+') /* positive result */
            return true;
        throw new SQLCacheD_Exception("Error: $resp");
    }


    function set_expiry_ops_interval($ops) {
        $ops = intval($ops);
        fwrite($this->sock, "EXI $ops\r\n");
        fflush($this->sock);
        $resp = trim(fgets($this->sock));
        if ($resp[0] == '+') /* positive result */
            return true;
        throw new SQLCacheD_Exception("Error: $resp");
    }


    function query_one($sql) {
        $a = $this->query($sql);
        if (is_array($a))
            return $a[0];
        return $a;
    }


    function get_ver() {
        fwrite($this->sock, "VER ".SQLCacheD_Client::SC_VER_SIG."\r\n");
        $resp = trim(fgets($this->sock));
        if (sscanf($resp, "+VER %s", $ver) === 1)
            return $ver;
        return false;
    }

}

?>
