# SQLCacheD Client for Python
# Copyrigh (c) 2006. Ivan Voras <ivoras@gmail.com>
# Released under the BSD License

# For some reason, Python client is even slower than PHP :(

import os, socket
import re


class SQLCacheD_Exception(Exception):
    pass


class SQLCacheD:

    DEFAULT_UNIX_SOCKET = '/tmp/sqlcached.sock'
    SC_VER_SIG = 'sqlcached-1'
    SOCK_UNIX = 'unix'
    SOCK_TCP = 'tcp'
    
    re_rec = re.compile(r"\+REC (\d+), (\d+)")
    re_nrec = re.compile(r"\+NREC (\d+)")
    re_ok = re.compile(r"\+OK (.+)")
    re_ver = re.compile(r"\+VER (.+)")

    def __init__(self, host = '/tmp/sqlcached.sock', type = 'unix'):
        if type != SQLCacheD.SOCK_UNIX:
            raise
        
        self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self.sock.connect(host)
        self.sf = self.sock.makefile('U', 4000)

        self.sf.write("VER %s\r\n" % SQLCacheD.SC_VER_SIG)
        self.sf.flush()
        if self.sf.readline().rstrip() != '+VER %s' % SQLCacheD.SC_VER_SIG:
            raise SQLCacheD_Exception("Handshake failure (invalid version signature?)")


    def query(self, sql):
        """SQL must not be unicode"""
        self.sf.write("SQL %s\r\n" % sql)
        self.sf.flush()
        resp = self.sf.readline().rstrip()
        m = SQLCacheD.re_rec.match(resp)
        if m != None: # only if some rows are returned (SELECT)
            n_rows = int(m.group(1))
            n_cols = int(m.group(2))
            cols = []
            for c in xrange(n_cols):
                cols.append(self.sf.readline().rstrip())
            rs = []
            for r in xrange(n_rows):
                row = {}
                for c in xrange(n_cols):
                    row[cols[c]] = self.sf.readline().rstrip()
                rs.append(row)
            return rs
        m = SQLCacheD.re_nrec.match(resp)
        if m != None: # number of rows
            return int(m.group(1))
        raise SQLCacheD_Exception(resp)
        

    def query_one(self, sql):
        rs = self.query(sql)
        return rs[0]
    

    def get_ver(self):
        self.sf.write("VER %s\r\n" % SQLCacheD.SC_VER_SIG)
        self.sf.flush()
        resp = self.sf.readline().rstrip()
        m = SQLCacheD.re_ver.match(resp)
        if m != None:
            return m.group(1)
        return False

        
